//
//    TIOSImageRequest.m
//    TWiOS5PhotoUploadExample
//
//    Copyright (c) 2012 Sean Cook (@theSeanCook)
//
//    Permission is hereby granted, free of charge, to any person obtaining a copy
//    of this software and associated documentation files (the "Software"), to deal
//    in the Software without restriction, including without limitation the rights
//    to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
//    copies of the Software, and to permit persons to whom the Software is
//    furnished to do so, subject to the following conditions:
//
//    The above copyright notice and this permission notice shall be included in
//    all copies or substantial portions of the Software.
//
//    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
//    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
//    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
//    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
//    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
//    OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
//    THE SOFTWARE.
//

#import "TIOSImageRequest.h"
#import <Twitter/Twitter.h>
#import <Accounts/Accounts.h>

#define ENDPOINT_MEDIA_UPLOAD @"https://upload.twitter.com/1/statuses/update_with_media.json"
#define CONTENT_TYPE_MULTIPART_FORM_DATA @"multipart/form-data"
#define PARAM_MEDIA @"media[]"
#define PARAM_STATUS @"status"
#define KEY_IMAGE @"image"
#define KEY_STATUS @"status"

@implementation TIOSImageRequest

@synthesize image = _image;
@synthesize status = _status;

#pragma mark - Class methods

+ (TIOSImageRequest *)imageRequest
{
    return [[self alloc] initWithURL:[NSURL URLWithString:ENDPOINT_MEDIA_UPLOAD] parameters:nil requestMethod:TWRequestMethodPOST];
}

#pragma mark - instance methods

- (void)performRequestWithHandler:(TWRequestHandler)handler
{
    if (_image) {
        NSData *imageData = UIImagePNGRepresentation(_image);
        if (imageData) {
            [self addMultiPartData:imageData withName:PARAM_MEDIA type:CONTENT_TYPE_MULTIPART_FORM_DATA];
        }
    }

    if (_status) {
        [self addMultiPartData:[_status dataUsingEncoding:NSUTF8StringEncoding] withName:PARAM_STATUS type:CONTENT_TYPE_MULTIPART_FORM_DATA];
    }

    [super performRequestWithHandler:handler];
}

@end
