//
//    TIOSViewController.m
//    TWiOS5PhotoUploadExample
//
//    Copyright (c) 2012 Sean Cook (@theSeanCook)
//
//    Permission is hereby granted, free of charge, to any person obtaining a copy
//    of this software and associated documentation files (the "Software"), to deal
//    in the Software without restriction, including without limitation the rights
//    to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
//    copies of the Software, and to permit persons to whom the Software is
//    furnished to do so, subject to the following conditions:
//
//    The above copyright notice and this permission notice shall be included in
//    all copies or substantial portions of the Software.
//
//    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
//    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
//    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
//    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
//    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
//    OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
//    THE SOFTWARE.
//

#import "TIOSViewController.h"
#import "TIOSImageRequest.h"
#import <Twitter/Twitter.h>
#import <Accounts/Accounts.h>
#import <AssetsLibrary/AssetsLibrary.h>

@interface TIOSViewController()
- (void)uploadUsingTweetSheet:(id)sender;
- (void)uploadUsingTWRequest:(id)sender;
- (void)_performTweetSheetUploadWithImage:(UIImage *)image;
- (void)_performTWRequestUploadWithImage:(UIImage *)image;
- (void)_enableButtons:(BOOL)enabled;
@end

@implementation TIOSViewController
{
    UIActivityIndicatorView *_activityView;
    UILabel *_accountLabel;
    UILabel *_statusLabel;
    UIButton *_btnUploadPhotoViaApi;
    UIButton *_btnUploadPhotoTweetSheet;
    ACAccountStore *_accountStore;
    TWPhotoExample _exampleType;
    NSArray *_accounts;
}

- (void)loadView
{
    UIView *theView = [[UIView alloc] initWithFrame:CGRectMake(0.0, 20.0, 320.0, 460.0)];
    theView.autoresizesSubviews = YES;
    theView.autoresizingMask = UIViewAutoresizingFlexibleWidth | UIViewAutoresizingFlexibleHeight;
    theView.backgroundColor = [UIColor colorWithRed:0.600 green:0.600 blue:0.600 alpha:1.000];
    theView.frame = CGRectMake(0.0, 20.0, 320.0, 460.0);
    self.view = theView;

    UIImageView *logoImageView = [[UIImageView alloc] initWithFrame:CGRectMake(79.0, 0.0, 150.0, 150.0)];
    logoImageView.autoresizingMask = UIViewAutoresizingFlexibleRightMargin | UIViewAutoresizingFlexibleBottomMargin;
    logoImageView.contentMode = UIViewContentModeScaleAspectFit;
    logoImageView.frame = CGRectMake(79.0, 0.0, 150.0, 150.0);
    logoImageView.userInteractionEnabled = NO;
    logoImageView.image = [UIImage imageNamed:@"twitter.png"];

    [theView addSubview:logoImageView];

    _statusLabel = [[UILabel alloc] initWithFrame:CGRectMake(40.0, 380.0, 245.0, 21.0)];
    _statusLabel.frame = CGRectMake(40.0, 380.0, 245.0, 21.0);
    _statusLabel.text = @"";
    _statusLabel.textAlignment = UITextAlignmentCenter;
    _statusLabel.backgroundColor = [UIColor clearColor];

    _btnUploadPhotoTweetSheet = [UIButton buttonWithType:UIButtonTypeRoundedRect];
    _btnUploadPhotoTweetSheet.frame = CGRectMake(24.0, 187.0, 275.0, 37.0);
    [_btnUploadPhotoTweetSheet setTitle:@"Upload Photo with Tweet Sheet" forState:UIControlStateNormal];
    [_btnUploadPhotoTweetSheet addTarget:self action:@selector(uploadUsingTweetSheet:) forControlEvents:UIControlEventTouchUpInside];

    _btnUploadPhotoViaApi = [UIButton buttonWithType:UIButtonTypeRoundedRect];
    _btnUploadPhotoViaApi.frame = CGRectMake(25.0, 244.0, 275.0, 37.0);
    [_btnUploadPhotoViaApi setTitle:@"Upload Photo with TWRequest" forState:UIControlStateNormal];
    [_btnUploadPhotoViaApi addTarget:self action:@selector(uploadUsingTWRequest:) forControlEvents:UIControlEventTouchUpInside];

    _activityView = [[UIActivityIndicatorView alloc] initWithActivityIndicatorStyle:UIActivityIndicatorViewStyleWhiteLarge];
    _activityView.frame = CGRectMake(143.0, 329.0, 37.0, 37.0);
    _activityView.hidden = YES;
    _activityView.hidesWhenStopped = YES;

    _accountLabel = [[UILabel alloc] initWithFrame:CGRectMake(25.0, 138.0, 275.0, 21.0)];
    _accountLabel.autoresizingMask = UIViewAutoresizingFlexibleRightMargin | UIViewAutoresizingFlexibleBottomMargin;
    _accountLabel.frame = CGRectMake(25.0, 138.0, 275.0, 21.0);
    _accountLabel.text = @"";
    _accountLabel.textAlignment = UITextAlignmentCenter;
    _accountLabel.backgroundColor = [UIColor clearColor];

    [theView addSubview:_activityView];
    [theView addSubview:_statusLabel];
    [theView addSubview:_btnUploadPhotoViaApi];
    [theView addSubview:_btnUploadPhotoTweetSheet];
    [theView addSubview:_accountLabel];
}

#pragma mark - Private

- (void)_uploadPhotoPushed:(TWPhotoExample)typeOfExample
{
    [self _enableButtons:NO];

    _exampleType = typeOfExample;

    if ([UIImagePickerController isSourceTypeAvailable:UIImagePickerControllerSourceTypePhotoLibrary]) {
        //  By default, we'll only be able to choose images, which is what we want
        UIImagePickerController *imagePickerController = [[UIImagePickerController alloc] init];
        imagePickerController.delegate = self;
        imagePickerController.sourceType = UIImagePickerControllerSourceTypePhotoLibrary;
        [self presentModalViewController:imagePickerController animated:YES];
    }
}

- (void)uploadUsingTweetSheet:(id)sender
{
    [self _uploadPhotoPushed:TWPhotoExampleWithTweetSheet];
}

- (void)uploadUsingTWRequest:(id)sender
{
    [self _uploadPhotoPushed:TWPhotoExampleUploadWithTWRequest];
}

- (void)_performTweetSheetUploadWithImage:(UIImage *)image
{
    NSAssert(image, @"Called without an image!");

    TWTweetComposeViewController *vc = [[TWTweetComposeViewController alloc] init];

    //  Sets the completion handler.  Note that we don't know which thread the block will be called on,
    //  so we must ensure that the UI updates occur on the main queue
    vc.completionHandler = ^(TWTweetComposeViewControllerResult result) {
        NSString *statusText = @"";
        if (result == TWTweetComposeViewControllerResultCancelled) {
            statusText = @"Tweet Sheet canceled";
        }
        else {
            statusText = @"Tweet Sheet completed!";;
        }

        dispatch_async(dispatch_get_main_queue(), ^{
            [self dismissViewControllerAnimated:NO completion:^{
                NSLog(@"Tweet Sheet has been dismissed.");
            }];
            _statusLabel.text = statusText;
            [_activityView stopAnimating];
            [self _enableButtons:YES];
        });
    };

    [vc setInitialText:@"This is a photo from my library #TWiOS5PhotoUploadExample"];

    if (![vc addImage:image]) {
        NSLog(@"Unable to add the image!");
    }

    //  Again, ensure that any UI actions occur on the main thread.
    dispatch_async(dispatch_get_main_queue(), ^{
        [self presentViewController:vc animated:NO completion:^{
            NSLog(@"Tweet sheet has been presented.");
        }];
    });
}

- (void)_performTWRequestUploadWithImage:(UIImage *)image
{
    NSAssert(image, @"Called without an image!");

    //  Uploads the selected image to the media upload API via a wrapped TWRequest
    TIOSImageRequest *req = [TIOSImageRequest imageRequest];

    //  Adds a status to the Tweet.  Note that the status gets added as multipart data in TWImageRequest.
    [req setStatus:@"This is a photo from my library, posted with TWRequest. #TWiOS5PhotoUploadExample"];
    [req setImage:image];
    [req setAccount:[_accounts objectAtIndex:0]];

    [req performRequestWithHandler:^(NSData *responseData, NSHTTPURLResponse *urlResponse, NSError *error) {
        if (!responseData) {
            NSLog(@"[ERROR]  URL Response:(%d) Error Message:(%@)", [urlResponse statusCode], error);
        }
        else {
            NSError *jsonError;
            id output = [NSJSONSerialization JSONObjectWithData:responseData options:NSJSONReadingMutableLeaves error:&jsonError];
            NSLog(@"%@", output);
        }

        dispatch_async(dispatch_get_main_queue(), ^{
            _statusLabel.text = @"TWRequest upload completed!";
            [_activityView stopAnimating];
            [self _enableButtons:YES];
        });
    }];
}


- (void)_enableButtons:(BOOL)enabled
{
    _btnUploadPhotoViaApi.enabled = enabled;
    _btnUploadPhotoTweetSheet.enabled = enabled;
    _btnUploadPhotoViaApi.alpha = (enabled) ? 1.0f : 0.5f;
    _btnUploadPhotoTweetSheet.alpha = (enabled) ? 1.0f : 0.5f;
}

#pragma mark - UIImagePickerControllerDelegate

- (void)_loadTwitterAccounts
{
    if (![TWTweetComposeViewController canSendTweet]) {
        UIAlertView *alert = [[UIAlertView alloc] initWithTitle:@"Yikes!" message:@"You must configure a Twitter account in Settings.app\nto use this demo." delegate:nil cancelButtonTitle:@"OK" otherButtonTitles:nil];
        [alert show];
    }
    else {
        ACAccountType *twitterAcctType = [_accountStore accountTypeWithAccountTypeIdentifier:ACAccountTypeIdentifierTwitter];
        [_accountStore requestAccessToAccountsWithType:twitterAcctType withCompletionHandler:^(BOOL granted, NSError *error) {
            if (granted) {
                _accounts = [_accountStore accountsWithAccountType:twitterAcctType];
                NSString *username = [[_accounts objectAtIndex:0] username];

                // make sure the user has accounts
                if (username != nil) {
                    dispatch_async(dispatch_get_main_queue(), ^{
                        _accountLabel.text = [NSString stringWithFormat:@"Using account: @%@", username];
                        [self _enableButtons:YES];
                    });
                }
                else {
                    NSLog(@"There are no Twitter accounts available.");
                }
            }
            else {
                NSLog(@"The user rejected your advances.");
                NSLog(@"Error: %@", error);
            }
        }];
    }
}

#pragma mark - UIImagePickerControllerDelegate

- (void)imagePickerController:(UIImagePickerController *)picker didFinishPickingMediaWithInfo:(NSDictionary *)info
{
    [picker dismissViewControllerAnimated:YES completion:^{
        UIImage *img = [info objectForKey:UIImagePickerControllerOriginalImage];
        NSAssert(img, @"Image wasn't returned from the picker?");

        //  Update our UI to indicate current status
        [_activityView startAnimating];
        [_statusLabel setText:@"Preparing image..."];
        [self _enableButtons:NO];

        switch(_exampleType) {
            case TWPhotoExampleWithTweetSheet:
                [self _performTweetSheetUploadWithImage:img];
                break;
            case TWPhotoExampleUploadWithTWRequest:
                [self _performTWRequestUploadWithImage:img];
                break;
        }
    }];
}

- (void)imagePickerControllerDidCancel:(UIImagePickerController *)picker
{
    [self dismissModalViewControllerAnimated:YES];
    [self _enableButtons:YES];
}

#pragma mark - View lifecycle

- (void)viewDidLoad
{
    [super viewDidLoad];

    //  The Account Store must live for at least as long as any of its ACAccount instances
    _accountStore = [[ACAccountStore alloc] init];

    [_statusLabel setText:@""];
    [self _enableButtons:NO];
    [self _loadTwitterAccounts];
}

- (void)viewDidUnload
{
    _accountLabel = nil;
    _statusLabel = nil;
    _btnUploadPhotoViaApi = nil;
    _btnUploadPhotoTweetSheet = nil;
    [super viewDidUnload];
}

@end
